<?php

use Mbl\Membergroup\AccessNotFoundError;

add_action( 'wpm_update_user_key_dates', 'tg_group_action_wpm_levels', 10, 3 );
function tg_group_action_wpm_levels( $user_id, $wpmlevel_id ): void {
	if ( get_term_meta( $wpmlevel_id, 'tg_group_on', true ) === 'yes' ) {
		$inviteLink = get_term_meta( $wpmlevel_id, 'tg_group_invite_link', true );
		$chat_id    = get_field( 'chat_id', 'user_' . $user_id );
		$group_id   = get_term_meta( $wpmlevel_id, 'tg_group_chat', true );
		$userGroups = get_user_meta( $user_id, 'tg_groups_in', true ) ?: [];
		if ( ! in_array( $group_id, $userGroups ) ) {
			$text   = get_term_meta( $wpmlevel_id, 'tg_group_message', true );
			$img_id = (int) get_term_meta( $wpmlevel_id, 'tg_group_image', true );
		} else {
			$text   = get_term_meta( $wpmlevel_id, 'tg_group_message_second', true );
			$img_id = (int) get_term_meta( $wpmlevel_id, 'tg_group_image_second', true );
		}
		$message = str_replace( '[membergroup_invitation]', $inviteLink, $text );
		$image   = wp_get_attachment_image_url( $img_id );
		if ( $image !== false ) {
			tg_send_photo( $chat_id, $image, $message );
		} else {
			tg_send( $chat_id, $message );
		}
	}
}

add_action( 'tg_webhook_action', 'mbg_webhook_action_callback' );
function mbg_webhook_action_callback( $data ): void {
	try {
		$info = tg_request( 'getMe' );
		if ( isset( $data['my_chat_member'] ) ) {
			$chat_id  = $data['my_chat_member']['new_chat_member']['user']['id'];
			$status   = $data['my_chat_member']['new_chat_member']['status'];
			$group_id = $data['my_chat_member']['chat']['id'];
			$title    = $data['my_chat_member']['chat']['title'];
			$chats    = get_option( 'tg_admin_chats' ) ?: [];
			if ( $chat_id === $info->result->id ) {
				if ( $status === 'administrator' ) {
					$chats[ $group_id ] = [
						'chat_id'      => $group_id,
						'title'        => $title,
						'bot'          => $info->result->id,
						'bot_name'     => $info->result->first_name,
						'bot_username' => $info->result->username,
					];
				} else {
					unset( $chats[ $group_id ] );
				}
				update_option( 'tg_admin_chats', $chats, false );
			}
		} elseif ( isset( $data['chat_join_request'] ) ) {
			$group_id    = $data['chat_join_request']['chat']['id'];
			$user_id     = $data['chat_join_request']['user_chat_id'];
			$invite_link = $data['chat_join_request']['invite_link'];
			$user        = mbg_user_by_tg_user_id( $user_id );
			try {
				$access = mbg_user_access( $user, $invite_link );
				tg_request( 'approveChatJoinRequest', [ 'chat_id' => $group_id, 'user_id' => $user_id ] );
				do_action( 'mbg_chat_join', $user, $access, $group_id, $user_id );
			} catch ( AccessNotFoundError  $exception ) {
				tg_request( 'declineChatJoinRequest', [ 'chat_id' => $group_id, 'user_id' => $user_id ] );
				do_action( 'mbg_decline_chat_join', $user, $group_id, $invite_link );
			}
		}
	} catch ( Exception $exception ) {
	}
}

add_action( 'mbg_chat_join', 'mbg_on_chat_join', 10, 4 );
function mbg_on_chat_join( WP_User $user, WP_Term $access, $group_id, $user_chat_id ): void {
	$joins = get_user_meta( $user->ID, 'tg_group_joins' );
	if ( empty( $joins ) ) {
		add_user_meta( $user->ID, 'tg_group_joins', $group_id );
	} else {
		if ( ! in_array( $group_id, $joins ) ) {
			add_user_meta( $user->ID, 'tg_group_joins', $group_id );
		}
	}
	$userGroups = get_user_meta( $user->ID, 'tg_groups_in', true );
	if ( empty( $userGroups ) ) {
		$userGroups = [ $group_id ];
	} else {
		if ( ! in_array( $group_id, $userGroups ) ) {
			$userGroups[] = $group_id;
		}
	}
	update_user_meta( $user->ID, 'tg_groups_in', $userGroups );

	tg_send( $user_chat_id, 'Заявка одобрена' );
}

add_action( 'mbg_schedule_event', 'mbg_schedule_event' );
function mbg_schedule_event(): void {
	global $wpdb;
	$now   = new DateTime( 'now', wp_timezone() );
	$users = $wpdb->get_results( $wpdb->prepare( "SELECT usermeta.user_id AS wp_user_id, usermeta2.meta_value AS tg_user_id, 
        usermeta.meta_value AS chat_id, MAX(t_keys.date_end) AS date_end
        FROM `$wpdb->usermeta` AS usermeta
        LEFT JOIN `$wpdb->termmeta` AS access
            ON access.meta_value = usermeta.meta_value AND access.meta_key = 'tg_group_chat'
        LEFT JOIN `{$wpdb->prefix}memberlux_term_keys` AS t_keys
            ON t_keys.term_id = access.term_id AND t_keys.user_id = usermeta.user_id AND t_keys.is_banned = 0
        LEFT JOIN `$wpdb->termmeta` AS access_meta
	        ON access_meta.meta_key = 'tg_group_exclude' AND access_meta.term_id = access.term_id
        LEFT JOIN `$wpdb->usermeta` AS usermeta2
            ON usermeta2.user_id = usermeta.user_id AND usermeta2.meta_key = 'chat_id'
        WHERE usermeta.meta_key = 'tg_group_joins' AND access_meta.meta_value = 'yes'
        GROUP BY usermeta.user_id, usermeta.meta_value
        HAVING MAX(t_keys.date_end) < %s;"
		, [ $now->format( 'Y-m-d' ) ] ), ARRAY_A );
	foreach ( $users as $user ) {
		$chat_id    = (int) $user['chat_id'];
		$wp_user_id = $user['wp_user_id'];
		$term_id    = (int) $wpdb->get_var( $wpdb->prepare( "SELECT t_keys.term_id
			FROM `{$wpdb->prefix}memberlux_term_keys` AS t_keys
			LEFT JOIN `$wpdb->termmeta` AS access
				ON t_keys.term_id = access.term_id AND access.meta_key = 'tg_group_chat'
			LEFT JOIN `$wpdb->termmeta` AS access_meta
	        	ON access_meta.meta_key = 'tg_group_exclude' AND access_meta.term_id = access.term_id
			WHERE t_keys.user_id = %d AND access.meta_value = %s AND t_keys.date_end = %s AND t_keys.is_banned = 0
				 AND access_meta.meta_value = 'yes'
			ORDER BY t_keys.id DESC
			LIMIT 1
		", [ $wp_user_id, $chat_id, $user['date_end'] ] ) );
		$args       = [ 'chat_id' => $chat_id, 'user_id' => $user['tg_user_id'], 'revoke_messages' => false ];
		$offset     = absint( get_term_meta( $term_id, 'tg_group_offset', true ) );
		if ( $offset ) {
			$dateNow = new DateTime( ( new DateTime( 'now', wp_timezone() ) )->format( 'Y-m-d 00:00:00' ), wp_timezone() );
			$dateEnd = new DateTime( "{$user['date_end']} 00:00:00", wp_timezone() );
			$dateEnd->modify( "+{$offset} day" );
			if ( $dateEnd->getTimestamp() >= $dateNow->getTimestamp() ) {
				continue;
			}
		}
		tg_request( 'banChatMember', $args );
		do_action( 'mbg_baned_chat_member', $wp_user_id, $chat_id );
	}
}

add_action( 'wpm_mbl_term_key_updated', 'mbg_mbl_term_key_updated', 10, 1 );
function mbg_mbl_term_key_updated( $key ): void {
	if ( empty( $key['user_id'] ) ) {
		return;
	}
	if ( $key['is_banned'] !== 1 ) {
		return;
	}
	global $wpdb;
	$now        = new DateTime( 'now', wp_timezone() );
	$wp_user_id = (int) $key['user_id'];
	$term_id    = (int) $key['term_id'];
	$tg_user_id = get_field( 'chat_id', 'user_' . $wp_user_id );
	$group_id   = get_term_meta( $term_id, 'tg_group_chat', true );
	$keys       = $wpdb->get_results( $wpdb->prepare( "
		SELECT termmeta2.meta_value AS group_id, term_keys.date_end, term_keys.term_id AS term_id 
		FROM `{$wpdb->prefix}memberlux_term_keys` AS term_keys 
		LEFT JOIN `$wpdb->termmeta` AS termmeta 
		    ON termmeta.term_id = term_keys.term_id AND termmeta.meta_key = 'tg_group_on' 
	    LEFT JOIN `$wpdb->termmeta` AS termmeta2 
	        ON termmeta2.term_id = term_keys.term_id AND termmeta2.meta_key = 'tg_group_chat' 
	    WHERE user_id = %d AND date_end >= %s  AND termmeta2.meta_value = %s
	      AND termmeta.meta_value = 'yes' AND term_keys.is_banned = 0	    	
		ORDER BY term_keys.date_end DESC
		"
		, [ $wp_user_id, $now->format( 'Y-m-d' ), $group_id ] ), ARRAY_A );
	if ( ! empty( $keys ) ) {
		return;
	}
	$args = [ 'chat_id' => $group_id, 'user_id' => $tg_user_id, 'revoke_messages' => false ];
	tg_request( 'banChatMember', $args );
	do_action( 'mbg_baned_chat_member', $wp_user_id, $group_id );
}

add_action( 'tg_connected_telegram', 'mbg_tg_connected_telegram_callback', 10, 1 );
function mbg_tg_connected_telegram_callback( $wordpress_user_id ): void {
	global $wpdb;
	$now   = new DateTime( 'now', wp_timezone() );
	$keys  = $wpdb->get_results( $wpdb->prepare( "
		SELECT termmeta2.meta_value AS group_id, term_keys.date_end, term_keys.term_id AS term_id 
		FROM `{$wpdb->prefix}memberlux_term_keys` AS term_keys 
		LEFT JOIN `$wpdb->termmeta` AS termmeta 
		    ON termmeta.term_id = term_keys.term_id AND termmeta.meta_key = 'tg_group_on' 
	    LEFT JOIN `$wpdb->termmeta` AS termmeta2 
	        ON termmeta2.term_id = term_keys.term_id AND termmeta2.meta_key = 'tg_group_chat' 
	    WHERE user_id = %d AND termmeta.meta_value = 'yes' AND date_end >= %s AND term_keys.is_banned = 0
		ORDER BY term_keys.date_end DESC
		"
		, [ $wordpress_user_id, $now->format( 'Y-m-d' ) ] ), ARRAY_A );
	$chats = [];
	foreach ( $keys as $key ) {
		if ( ! in_array( $key['group_id'], $chats ) ) {
			$chats[] = $key['group_id'];
			$term_id = $key['term_id'];
			tg_group_action_wpm_levels( $wordpress_user_id, $term_id );
		}
	}
}

add_action( 'mbg_baned_chat_member', 'mbg_after_ban_chat_member', 10, 2 );
function mbg_after_ban_chat_member( $wp_user_id, $chat_id ): void {
	global $wpdb;
	$wpdb->delete( $wpdb->usermeta, [
		'user_id'    => $wp_user_id,
		'meta_key'   => 'tg_group_joins',
		'meta_value' => $chat_id
	] );
}