<?php

use Mbl\Membergroup\AccessNotFoundError;
use Mbl\Membergroup\Chat;

/**
 * @throws AccessNotFoundError
 */
function mbg_user_access(WP_User $user, $invite_link): WP_Term
{
    global $wpdb;
    $terms = get_terms([
        'taxonomy' => 'wpm-levels',
        'hide_empty' => false,
        'meta_query' => [['key' => 'tg_group_invite_link', 'value' => $invite_link]],
    ]);
    if (empty($terms)) {
        throw new AccessNotFoundError();
    }
    $term = $terms[0];
    $keys = $wpdb->get_results($wpdb->prepare(
        "SELECT id FROM `{$wpdb->prefix}memberlux_term_keys`
               WHERE user_id = %d AND term_id = %d AND is_banned = 0 AND date_end >= %s"
        , [$user->ID, $term->term_id, date('Y-m-d')]));
    if (empty($keys)) {
        throw new AccessNotFoundError();
    }

    return $term;
}

/**
 * @throws Exception
 */
function mbg_user_by_tg_user_id($user_id): WP_User
{
    $persons = get_posts(array(
        'post_type' => 'tg_person',
        'meta_key' => 'chat_id',
        'meta_value' => $user_id
    ));
    if (empty($persons)) {
        throw new Exception("User telegram user_id=$user_id not found");
    }
    $person_id = $persons[count($persons) - 1]->ID;
    $person_email = get_field('tg_email', $person_id);
    $user = get_user_by('email', $person_email);
    if ($user === false) {
        throw new Exception("User telegram user_id=$user_id not found");
    }

    return $user;
}

function mbg_get_chats(): array
{
    if (empty(get_option('tg_admin_chats'))) {
        $return = [];
    } else {
        $return = array_map(fn($chatId) => new Chat($chatId), array_keys(get_option('tg_admin_chats')));
    }

    return $return;
}